package com.application.DB;

import com.google.auth.oauth2.GoogleCredentials;
import com.google.auth.oauth2.ServiceAccountCredentials;
import com.google.cloud.bigquery.*;

import java.io.File;
import java.io.FileInputStream;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAccessor;
import java.util.*;

/**
 * This class is responsible for handling database related activities
 *
 * @author Eilert Tunheim, Karin Pettersen, Mads Arnesen
 * @version 1.0
 */
public class DB {

    private static Job queryJob;
    private static SimpleDateFormat dateFormat;

    /**
     * Retrieves the credentials file
     *
     * @return the credentials
     * @throws Exception for potential errors
     */
    private static GoogleCredentials getCredentials() throws Exception {
        File credentialsPath = new File("./src/main/resources/com.application/sf-drying-optimization-1e234ad2b0f4.json");

        // Load credentials from JSON key file. If you can't set the GOOGLE_APPLICATION_CREDENTIALS
        // environment variable, you can explicitly load the credentials file to construct the
        // credentials.
        GoogleCredentials credentials;
        try (FileInputStream serviceAccountStream = new FileInputStream(credentialsPath)) {
            credentials = ServiceAccountCredentials.fromStream(serviceAccountStream);
        }
        return credentials;
    }

    public static void main(String[] args) throws Exception {
        getKwh();
    }


    /**
     * Creates a builder
     *
     * @return a builder
     * @throws Exception returns potential error
     */
    private static BigQuery getBuilder() throws Exception {
        // Step 1: Initialize BigQuery service
        // Here we set our project ID and get the `BigQuery` service object
        // this is the interface to our BigQuery instance that
        // we use to execute jobs on
        return BigQueryOptions.newBuilder().
                setCredentials(getCredentials()).
                setProjectId("sf-drying-optimization")
                .build().getService();
    }


    /**
     * Creates a job
     *
     * @param queryConfig query configuration information
     * @return a job
     * @throws Exception returns potential error
     */
    private static Job getJob(JobConfiguration queryConfig) throws Exception {
        // Step 3: Run the job on BigQuery
        // create a `Job` instance from the job configuration using the BigQuery service
        // the job starts executing once the `create` method executes

        Job queryJob = getBuilder().create(JobInfo.newBuilder(queryConfig).build());
        queryJob = queryJob.waitFor();
        // the waitFor method blocks until the job completes
        // and returns `null` if the job doesn't exist anymore
        if (queryJob == null) {
            throw new Exception("job no longer exists");
        }
        // once the job is done, check if any error occured
        if (queryJob.getStatus().getError() != null) {
            throw new Exception(queryJob.getStatus().getError().toString());
        }
        return queryJob;
    }


    /**
     * Retrieves information about kwh and the corresponding date
     *
     * @return the results
     * @throws Exception returns potential error
     */
    public static Map<String, Number> getKwh() throws Exception {

        // Initializing the data map to store the results
        Map<String, Number> data = new HashMap<>();

        // Initializing the date format
        DateTimeFormatter format = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        // Preparing a query statement
        final String sqlStatement = "SELECT TimeStamp, VariantValue " +
                "FROM sf-drying-optimization.124.int_sd_winccsensordata " +
                "WHERE TimeStamp BETWEEN \"2021-01-25 08:51:03\" " +
                "AND \"2021-02-08 11:10:09\" ORDER BY TimeStamp ASC";

        // Creates a job configuration
        queryJob = getJob(QueryJobConfiguration.newBuilder(sqlStatement).build());

        // Iterating through the results
        TableResult result = queryJob.getQueryResults();
        System.out.println("InTidTork \t UtTidTork");
        for (FieldValueList row : result.iterateAll()) {

            // Kwh
            int variantValue = row.get("VariantValue").getNumericValue().intValue();

            // Retrieving the wanted data
            long timeStamp = row.get("TimeStamp").getTimestampValue()/1000;
            // Riktig format, men i string
            String formatedTimeStamp = dateFormat.format(timeStamp);
            // Format the date to LocalDateTime datatype
            LocalDateTime date = LocalDateTime.parse(formatedTimeStamp, format);
            // Creates a future date to compare against
            LocalDateTime dateFuture = date.plusDays(1);

            //System.out.println(date);

            data.put(formatedTimeStamp, variantValue);
        }
/*
        NavigableMap<String, Number> sortedData = new TreeMap<>(data);

        for (Map.Entry<String, Number> entry : sortedData.entrySet()) {
            String key = entry.getKey();
            String next = sortedData.higherEntry(entry.getKey()).getKey(); // next

            System.out.println("Nå: "+key);
            System.out.println("Neste: "+next);
        }

 */


            return data;
    }

    /**
     * Retrieves information about kwh and the corresponding date
     *
     * @return the results
     * @throws Exception returns potential error
     */
    public static void getName() throws Exception {

        // Initializing the data map to store the results
        Map<Object,ArrayList<Object>> data = new HashMap<>();


        // Initializing the date format
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        dateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));



        // Preparing a query statement
        final String sqlStatement =
                        "SELECT Name, CalculatedStart, CalculatedStop FROM `sf-drying-optimization.124.int_dk_valmaticsdryingbatches`" +
                        "WHERE Name Like \"%Gran%\" AND Name Like \"%3ex%\" AND NAME Like \"%47x150%\" AND DATE(CalculatedStart) " +
                        "BETWEEN \"2018-08-17\" AND \"2022-08-30\" ORDER BY DATE(CalculatedStart)";

        // Creates a job configuration
        queryJob = getJob(QueryJobConfiguration.newBuilder(sqlStatement).build());

        // Iterating through the results
        TableResult result = queryJob.getQueryResults();
        System.out.println("Name\tCalculatedStarted\tCalculatedStop");
        for (FieldValueList row : result.iterateAll()) {

            // Retrieving the wanted data
            String name = row.get("Name").getStringValue();
            // The dates are returned as a 16-digit number that needs to be formatted
            long calculatedStart = row.get("CalculatedStart").getTimestampValue()/1000;
            long calculatedStop = row.get("CalculatedStop").getTimestampValue()/1000;
            // Formatting the dates
            String formattedCalculatedStart = dateFormat.format(calculatedStart);
            String formattedCalculatedStop = dateFormat.format(calculatedStop);

            java.sql.Timestamp timestamp = new Timestamp(calculatedStart);
            //System.out.println(timestamp);

            //System.out.printf("%s\t\t\t%s\t\t\t%s\n",name, formattedCalculatedStart, formattedCalculatedStop);

        }
        //return data;
    }
}